library(raster)
library(sp)
library(rgdal)
library(rgeos)
library(spatstat)
library(maptools)
library(dismo)
library(gstat)
library(spdep)

#1.	Ouvrir le fichier  BDMINES.txt  (commande read.table, attention  bien configurer les arguments sep, dec et header).
bdmine <- read.table("BDMINEPB.txt", sep="\t", dec=",",header=TRUE)

#2. transformer le tableau en un objet de classe SpatialPointsDataFrame pour une future utilisation  (commande SpatialPointsDataFrame)
bd<-SpatialPointsDataFrame(bdmine[,2:3], bdmine, coords.nrs = numeric(0), proj4string = CRS("+init=epsg:27572") , match.ID = TRUE)

# 3.	Extraire les coordonnes de bd (commande coordinates)
xy <- coordinates(bd)

#4.	Quelle est la dimension de xy ? (commande dim)
dim(xy)

# 5. Eliminer les ventuels doublons
xy <- unique(xy)
dim(xy)
head(xy)

#6.	Reprsenter graphiquement la rpartition spatiale des mines de plomb (commande plot). Ajouter asp=1 dans les arguments de la commande plot. Quel est l'impact de asp=1 ?
plot(xy)
plot(xy, asp=1)

# 7.	Ouvrir les limites du PNRM, d'une zone de 30x30 km2 autour de Bibracte, et la position de Bibracte ; toutes ces informations sont fournies sous forme de shapefiles.
LimitePNRM <- readOGR(dsn = ".", layer = "LimitePNRM")
LimiteBib <- readOGR(dsn = ".", layer = "LimiteZ1")
Bibracte<- readOGR(dsn = ".", layer = "Bibracte")
crs(LimitePNRM) <- "+init=epsg:27572"
crs(LimiteBib) <- "+init=epsg:27572"
crs(Bibracte) <- "+init=epsg:27572"

#8.	Reprsenter graphiquement ces informations, ainsi que la position des mines sur un mme graphique (commande plot et points)
plot(LimitePNRM, col="light blue")
plot(LimiteBib, col="yellow", add=T)
points(xy,col="darkblue", pch=15)
plot(Bibracte, add=T, col="white", cex=2, pch=16)


#9. selectionner les mines de Pb a l'interieur de la limite du parc, et les representer graphiquement de meme que la limite du parc
i <- gIntersects(bd, LimitePNRM, byid=TRUE)
int_pnrm<-xy[i,]

plot(LimitePNRM, col="light blue")
points(int_pnrm,col="darkblue", pch=15)
plot(Bibracte, add=T, col="white", cex=2, pch=16)

#10.	Slectionner les mines de Pb autour de Bibracte, et les reprsenter graphiquement de mme que la limite autour de Bibracte.
j <- gIntersects(bd, LimiteBib, byid=TRUE)
int_Bib<-xy[j,]

plot(LimiteBib, col="light blue")
points(int_Bib,col="darkblue", pch=15)
plot(Bibracte, add=T, col="white", cex=2, pch=16)

#11.	Calculer pour les mines autour de Bibracte, le centrode des mines et le cercle correspondant  la distance standard (voir cours)
mcb <- apply(int_Bib, 2, mean)
sdb <- sqrt(sum((int_Bib[,1] - mcb[1])^2 + (int_Bib[,2] - mcb[2])^2) / nrow(int_Bib))

plot(LimiteBib, col='light blue')
plot(Bibracte, add=T, col="white", cex=2, pch=16)
points(int_Bib, cex=1)
points(mcb[1],mcb[2], pch=16, cex=3, col="red")

bearing <- 1:360 * pi/180
cx <- mcb[1] + sdb * cos(bearing)
cy <- mcb[2] + sdb * sin(bearing)
circle <- cbind(cx, cy)
lines(circle, col='red', lwd=2)

#12.	Calculer l'aire de la zone autour de Bribracte (commande area) et la densit de mines par unit de surface.

ZoneBib <- area(LimiteBib)
densBib <- nrow(int_Bib) / ZoneBib
ZoneBib
densBib

# Analyse de motifs
# 13.	Crer une grille sur la zone d'intrt avec une rsolution de 5000 m, reportez y les mines, puis compter le nombre de cases possdant 0 mine, 1 mine, etc. 
r <- raster(LimiteBib)
res(r) <- 5000
r
r <- rasterize(LimiteBib, r)
plot(r)
quads <- as(r, 'SpatialPolygons')
plot(quads, add=TRUE)
points(int_Bib, col='red', cex=2)

#14.	Calculer le nombre moyen de mines par quadrat, la variance du nombre de mine par quadrat, puis le rapport variance/moyenne. Que peut-on conclure sur la distribution des mines  la mthode des quadrats (voir cours) ?
nc <- rasterize(coordinates(int_Bib), r, fun='count', background=0)
plot(nc)
plot(LimiteBib, add=TRUE)

f <- freq(nc, useNA='no')
f


quadrats <- sum(f[,2])
cases <- dim(int_Bib)[1]
mu <- cases / quadrats
mu

ff <- data.frame(f)
colnames(ff) <- c('K', 'X')
ff$Kmu <- ff$K - mu
ff$Kmu2 <- ff$Kmu^2
ff$XKmu2 <- ff$Kmu2 * ff$X
ff

s2 <- sum(ff$XKmu2) / (sum(ff$X)-1)
s2

VMR <- s2 / mu
VMR

# 15.	Calculer la matrice des distances entre les mines autour de Bibracte, slectionner les plus proches voisins, valuer la fonction G, et la reprsenter (voir cours)
d <- dist(int_Bib)
dm <- as.matrix(d)
diag(dm) <- NA
dmin <- apply(dm, 1, min, na.rm=TRUE)
mdmin <- mean(dmin)

distance <- sort(unique(round(dmin)))
Gd <- sapply(distance, function(x) sum(dmin < x))
Gd <- Gd / length(dmin)
plot(distance, Gd, type="l")

# 16.	Prendre les centres des mailles comme rfrence (commande rasterToPoints), les reprsenter graphiquement, calculer leur distance aux mines (commande pointDistance), calculer la fonction F, puis la reprsenter graphiquement.
p <- rasterToPoints(r)
plot(p, asp=1)
d2 <- pointDistance(p[,1:2], int_Bib, longlat=FALSE)
Fdistance <- sort(unique(round(d2)))
mind <- apply(d2, 1, min)
Fd <- sapply(Fdistance, function(x) sum(mind < x))
Fd <- Fd / length(mind)
plot(Fdistance, Fd, type='l', lwd=2)


# 17.	Crer une fonction correspondant  un processus de Poisson, puis reprsenter sur un mme diagramme en fonction de d ce qu'aurait produit ce processus, ainsi que les fonctions F et G (voir cours). Conclure.
ef <- function(d, lambda) {
  E <- 1 - exp(-1 * lambda * pi * d^2)
}
expected <- ef(0:4000, densBib)


plot(distance, Gd, type='l', lwd=2, col='red', las=1,
     ylab='F(d) or G(d)', xlab='Distance', yaxs="i", xaxs="i")
lines(Fdistance, Fd, lwd=2, col='blue')
lines(0:4000,expected, lwd=2)

legend(1200, .3,
       c(expression(italic("G")["d"]), expression(italic("F")["d"]), 'expected'),
       lty=1, col=c('red', 'blue', 'black'), lwd=2, bty="n")


#
#18.	Utiliser le package spatstat pour crer une carte de chaleur (kernel density map) des mines autour de Bibracte. D'abord dfinir la fentre d'observation (fonction as.owin), puis crer un motif de points (fonction ppp), enfin crer la carte de densit kernel (fonction density). Voir cours. 


Bibowin <- as.owin(LimiteBib)
Bibowin
pts <- coordinates(int_Bib)
p <- ppp(pts[,1], pts[,2], window=Bibowin)
plot(p)
plot(density(spp[1:4]), main='')

ds <- density(p)
plot(ds, main='mines de plomb autour de Bibracte')


#19.	Calculer le K de Ripley  l'aide du package spatstat (fonctions Kest et enveloppe ; voir cours).

spatstat.options(checksegments = FALSE)
Kbib <- Kest(p)
Kenv <- envelope(p, Kest)
plot(Kbib)
plot(Kenv)

#20.	Ouvrir le fichier "90km2new.txt" qui contient les rsultats des analyses du BRGM sur la zone autour de Bibracte. 

mtx <- read.table("90km2new.txt", sep="\t", dec=",",header=TRUE)
dsp <- SpatialPoints(mtx[,5:6], proj4string=CRS("+init=epsg:27572"))
dsp <- SpatialPointsDataFrame(dsp, mtx)